# client-rust 1.0 roadmap

The client-rust project complements the TiKV ecosystem by providing a reliable Rust client. It can be a foundation for implementing clients in other languages. There have been preliminary works in [client-py](https://github.com/tikv/client-py) and [client-cpp](https://github.com/tikv/client-cpp). There are also plans for general-purpose client-java and [client-node](https://github.com/tikv/tikv/issues/10054).

The document describes our plan to push client-rust towards its general availability (1.0).

## Deliverable

client-rust 1.0 that
- enables new users to start using it in 10 mins
- has been heavily tested under simulated and real workload
- is easy to use, debug and seek help from documentation
- is performant
- supports the most-wanted features: GC, async commit, etc.


## Milestones

There are several milestones when we make important progress. We may release new versions with these changes, but it depends. The version numbers are tentative and subject to change.

#### 0.1 - initial release

The initial release that is functional but may be weak in performance and user-friendliness. It supports: 
raw mode, including CAS
transactional mode, including optimistic and pessimistic

#### 0.2 - feature complete

0.2 should be feature complete 
- Region cache
- Large transaction
- Async commit
- (Optional) GC, depending on the progress of the new mechanism

#### 0.3 - user-friendly and high-quality

0.3 should be a relatively mature version. It is easy to use and have fewer bugs.

The tasks include better logging, error handling, compatibility with the Rust async ecosystem, better unit test and integration test.

#### 1.0 - production ready

The gap between 0.3 and 1.0 may include
- Documentation
- Test coverage
- Performance test / benchmark
- Usability improvement based on user feedback

## Estimated Timeline

The timeline is a rough estimate. It is subject to change if we find new issues, underestimate the workload, or have insufficient contributors’ time.

##### May 2021
Release 0.1.0

##### Aug 2021
Release 0.2.0

##### Oct 2021
Release 0.3.0

##### Jan 2022
Release 1.0

We will take a relative longer time to let users try the tikv-client, and improve based on their feedback.



## Risks

- Inaccurate estimated workload. Considering the complexity of the client in TiDB, we may underestimate the workload.
- Lacking staff. Regular contributors include Ziqian Qin and Andy Lok.
- Too much or too little external contribution - too much means core contributors don’t have time to focus on code and design, or that we add too many new features which do not get us to 1.0. Too little means we can’t iterate effectively based on user experience.


----

## Tasks

Tasks listed here are collected from https://github.com/tikv/client-rust/discussions/272. 

**Note**: They are preliminary, and we should adapt them according to user feedback.


### 0.2

- https://github.com/tikv/client-rust/issues/299 Add region cache
- https://github.com/tikv/client-rust/issues/209 Use CheckTxnStatus rather than Cleanup
- https://github.com/tikv/client-rust/issues/239 Better experience when "dropping active transaction"
- https://github.com/tikv/client-rust/issues/287 Support 1PC
- https://github.com/tikv/client-rust/issues/189 Support large transaction
- https://github.com/tikv/client-rust/issues/287 Support async commit

### 0.3


- https://github.com/tikv/client-rust/issues/267 Better logging
- https://github.com/tikv/client-rust/issues/284 Test with nemesis and fault injection
- https://github.com/tikv/client-rust/issues/246 Compatibility with any async reactor (don't bind to tokio)
- https://github.com/tikv/client-rust/issues/285 Improve unit test coverage
- https://github.com/tikv/client-rust/issues/286 TiKV version compatibility check
- https://github.com/tikv/client-rust/issues/284 Test with various workload: simulated / real / extreme

### 1.0


- https://github.com/tikv/client-rust/issues/288 Benchmarking script/tool
- https://github.com/tikv/client-rust/issues/283 Parallelize multi-region request
- https://github.com/tikv/client-rust/issues/289 Synchronous API



